==> rulee-ts-0.7.0.tgz: package/lib/esm/engine/builder/builder.js <==
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { PropertyScalarRuleBinding } from "./property-scalar-rule-binding";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { GroupOfPropertiesImpl } from "../../properties/group-of-properties-impl";
import { V } from "../../validators/common/common-validators";
import { ListOfPropertiesBuilder } from "./list-of-properties-builder";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
export class Builder {
    constructor(options, ruleEngine, dependencyGraph, propertyMap) {
        var _a;
        this.ruleEngine = ruleEngine;
        this.dependencyGraph = dependencyGraph;
        this.propertyMap = propertyMap;
        this.trigger = new TriggerBuilder();
        this.notEmptyIfRequiredValidator = options.emptyButRequiredMessage instanceof Function
            ? V.notEmptyMsgProvider(options.emptyButRequiredMessage)
            : V.notEmpty(options.emptyButRequiredMessage);
        this.defaultBackpressureConfig = Object.freeze((_a = options.defaultBackpressureConfig) !== null && _a !== void 0 ? _a : {
            type: "switch",
            debounceTime: 40
        });
        if (options.defaultEmptyChoiceDisplayValue) {
            this.defaultEmptyChoice = {
                value: null,
                displayValue: options.defaultEmptyChoiceDisplayValue
            };
        }
        this.scalar = new PropertyScalarBuilder((id, provider, emptyValueFcn, converter, dependencies, initialValue, backpressureConfig, ownedProperties) => this.propertyScalar(id, provider, emptyValueFcn, converter, dependencies, initialValue, backpressureConfig, ownedProperties), (prop) => this.bindPropertyScalar(prop), this.defaultEmptyChoice);
        this.group = new GroupOfPropertiesBuilder((id, properties, exportFcn, importFcn) => this.groupOfProperties(id, properties, exportFcn, importFcn));
        this.list = new ListOfPropertiesBuilder((id, listProvider, selectedIndices, isMultiSelect) => this.listOfProperties(id, listProvider, selectedIndices, isMultiSelect));
    }
    get properties() {
        return Object.values(this.propertyMap);
    }
    propertyScalar(id, provider, emptyValueFcn, converter, dependencies, initialValue, backpressureConfig, ownedProperties) {
        const prop = new PropertyScalarImpl(id, provider, emptyValueFcn, converter, this.ruleEngine, backpressureConfig !== null && backpressureConfig !== void 0 ? backpressureConfig : (provider.isAsynchronous() ? this.defaultBackpressureConfig : undefined));
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        if (ownedProperties) {
            ownedProperties.forEach(owned => this.dependencyGraph.addOwnerDependency(prop, owned, false));
        }
        if (initialValue !== undefined) {
            prop.defineInitialValue(initialValue);
        }
        prop.setToInitialValue();
        return prop;
    }
    bindPropertyScalar(prop) {
        return new PropertyScalarRuleBinding(prop, this.notEmptyIfRequiredValidator, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options));
    }
    groupOfProperties(id, properties, exportFcn, importFcn) {
        const prop = new GroupOfPropertiesImpl(id, properties, exportFcn, importFcn, this.ruleEngine);
        this.addProperty(prop);
        this.addDependencies(this.dependencyGraph, prop.propertiesAsList, prop, { value: true });
        return prop;
    }
    listOfProperties(id, listProvider, selectedIndices, isMultiSelect) {
        const prop = new ListOfPropertiesImpl(id, listProvider, selectedIndices, isMultiSelect, this.ruleEngine, this.dependencyGraph);
        this.addProperty(prop);
        return prop;
    }
    addDependencies(dependencyGraph, from, to, options) {
        if (from.length) {
            dependencyGraph.addDependencies(from, to, options);
        }
    }
    addProperty(property) {
        if (this.propertyMap[property.id]) {
            throw new Error(`A Property with the id ${property.id} is already defined`);
        }
        this.propertyMap[property.id] = property;
    }
    defineCustomAttribute(name) {
        return { name };
    }
    bindValidator(...properties) {
        return (validator) => {
            const instance = {
                getValidatedProperties: () => properties,
                validate: validator
            };
            properties.forEach((prop, i) => {
                prop.addValidator(instance);
                for (let j = i + 1; j < properties.length; j++) {
                    const dependent = properties[j];
                    this.dependencyGraph.addDependency(prop, dependent, { validation: true });
                    this.dependencyGraph.addDependency(dependent, prop, { validation: true });
                }
            });
        };
    }
    asVisJsData() {
        return this.dependencyGraph.createVisJsData(this.properties);
    }
    generateNetworkGraphHtmlPage() {
        const graph = this.asVisJsData();
        const nodes = JSON.stringify(graph.nodes);
        const edges = JSON.stringify(graph.edges);
        const options = JSON.stringify(graph.options);
        return `<!DOCTYPE HTML>
<html>
<head>
    <script src="https://visjs.github.io/vis-network/standalone/umd/vis-network.min.js"></script>
    <style>
        #network {
            position: absolute; top: 0; right: 0; bottom: 0; left: 0;
        }
    </style>
</head>
<body>
    <div id="network"></div>
    <script type="text/javascript">
        var container = document.getElementById("network");
        var data = {
            nodes: new vis.DataSet(${nodes}),
            edges: new vis.DataSet(${edges}),
        };
        var options = ${options};
        var network = new vis.Network(container, data, options);
    </script> 
</body>
</html>`;
    }
    generateFactory() {
        return '';
    }
}

==> rulee-ts-0.8.0.tgz: package/lib/esm/engine/builder/builder.js <==
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { PropertyScalarRuleBinding } from "./property-scalar-rule-binding";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { GroupOfPropertiesImpl } from "../../properties/group-of-properties-impl";
import { V } from "../../validators/common/common-validators";
import { ListOfPropertiesBuilder } from "./list-of-properties-builder";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
export class Builder {
    constructor(options, ruleEngine, dependencyGraph, propertyMap) {
        var _a;
        this.ruleEngine = ruleEngine;
        this.dependencyGraph = dependencyGraph;
        this.propertyMap = propertyMap;
        this.trigger = new TriggerBuilder();
        this.notEmptyIfRequiredValidator = options.emptyButRequiredMessage instanceof Function
            ? V.notEmptyMsgProvider(options.emptyButRequiredMessage)
            : V.notEmpty(options.emptyButRequiredMessage);
        this.defaultBackpressureConfig = Object.freeze((_a = options.defaultBackpressureConfig) !== null && _a !== void 0 ? _a : {
            type: "switch",
            debounceTime: 40
        });
        if (options.defaultEmptyChoiceDisplayValue) {
            this.defaultEmptyChoice = {
                value: null,
                displayValue: options.defaultEmptyChoiceDisplayValue
            };
        }
        this.scalar = new PropertyScalarBuilder((id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties) => this.propertyScalar(id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties), (prop) => this.bindPropertyScalar(prop), this.defaultEmptyChoice);
        this.group = new GroupOfPropertiesBuilder((id, properties, exportFcn, importFcn) => this.groupOfProperties(id, properties, exportFcn, importFcn));
        this.list = new ListOfPropertiesBuilder((id, listProvider, selectedIndices, isMultiSelect) => this.listOfProperties(id, listProvider, selectedIndices, isMultiSelect));
    }
    get properties() {
        return Object.values(this.propertyMap);
    }
    propertyScalar(id, provider, emptyValueFcn, converter, dependencies, config, ownedProperties) {
        var _a;
        const prop = new PropertyScalarImpl(id, provider, emptyValueFcn, converter, this.ruleEngine, (_a = config === null || config === void 0 ? void 0 : config.backpressure) !== null && _a !== void 0 ? _a : (provider.isAsynchronous() ? this.defaultBackpressureConfig : undefined));
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        if (ownedProperties) {
            ownedProperties.forEach(owned => this.dependencyGraph.addOwnerDependency(prop, owned, false));
        }
        if (config) {
            if (config.initialValue !== undefined) {
                prop.defineInitialValue(config.initialValue);
            }
            if (config.labelAndPlaceholder !== undefined) {
                prop.defineLabel(config.labelAndPlaceholder);
                prop.definePlaceholder(config.labelAndPlaceholder);
            }
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
            if (config.placeholder !== undefined) {
                prop.definePlaceholder(config.placeholder);
            }
        }
        prop.setToInitialValue();
        return prop;
    }
    bindPropertyScalar(prop) {
        return new PropertyScalarRuleBinding(prop, this.notEmptyIfRequiredValidator, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options));
    }
    groupOfProperties(id, properties, exportFcn, importFcn) {
        const prop = new GroupOfPropertiesImpl(id, properties, exportFcn, importFcn, this.ruleEngine);
        this.addProperty(prop);
        this.addDependencies(this.dependencyGraph, prop.propertiesAsList, prop, { value: true });
        return prop;
    }
    listOfProperties(id, listProvider, selectedIndices, isMultiSelect) {
        const prop = new ListOfPropertiesImpl(id, listProvider, selectedIndices, isMultiSelect, this.ruleEngine, this.dependencyGraph);
        this.addProperty(prop);
        return prop;
    }
    addDependencies(dependencyGraph, from, to, options) {
        if (from.length) {
            dependencyGraph.addDependencies(from, to, options);
        }
    }
    addProperty(property) {
        if (this.propertyMap[property.id]) {
            throw new Error(`A Property with the id ${property.id} is already defined`);
        }
        this.propertyMap[property.id] = property;
    }
    defineCustomAttribute(name) {
        return { name };
    }
    bindValidator(...properties) {
        return (validator) => {
            const instance = {
                getValidatedProperties: () => properties,
                validate: validator
            };
            properties.forEach((prop, i) => {
                prop.addValidator(instance);
                for (let j = i + 1; j < properties.length; j++) {
                    const dependent = properties[j];
                    this.dependencyGraph.addDependency(prop, dependent, { validation: true });
                    this.dependencyGraph.addDependency(dependent, prop, { validation: true });
                }
            });
        };
    }
    asVisJsData() {
        return this.dependencyGraph.createVisJsData(this.properties);
    }
    generateNetworkGraphHtmlPage() {
        const graph = this.asVisJsData();
        const nodes = JSON.stringify(graph.nodes);
        const edges = JSON.stringify(graph.edges);
        const options = JSON.stringify(graph.options);
        return `<!DOCTYPE HTML>
<html>
<head>
    <script src="https://visjs.github.io/vis-network/standalone/umd/vis-network.min.js"></script>
    <style>
        #network {
            position: absolute; top: 0; right: 0; bottom: 0; left: 0;
        }
    </style>
</head>
<body>
    <div id="network"></div>
    <script type="text/javascript">
        var container = document.getElementById("network");
        var data = {
            nodes: new vis.DataSet(${nodes}),
            edges: new vis.DataSet(${edges}),
        };
        var options = ${options};
        var network = new vis.Network(container, data, options);
    </script> 
</body>
</html>`;
    }
    generateFactory() {
        return '';
    }
}
